@@ -7,16 +7,18 @@ test_that("getMatchLabels works", {
     results <- data.frame(
         MatchId=c("g_6;em_12_419;n_9243-n_9245;p_14;#=ew_0_7260;[0]=ew_0_7260",
                   "g_6;em_12_429;n_9263-n_9265;p_14;#=ew_0_7487;[0]=ew_0_7487",
-                  "g_6;em_12_440;n_9285-n_9287;p_14;#=ew_0_7704;[0]=ew_0_7704"))
+                  "g_6;em_12_440;n_9285-n_9287;p_14;#=ew_0_7704;[0]=ew_0_7704",
+                  "g_25;em_12_1290;n_86196-n_86198;p_32;#=ew_0_26605;[0]=ew_0_26605"))
     
     ## get labels
     labels <- getMatchLabels(labbcat.url, results$MatchId, "topic")
 
-    expect_equal(length(labels$topic), 3)
+    expect_equal(length(labels$topic), 4)
     topic <- as.vector(labels$topic)
     expect_equal(topic[[1]], "{September earthquake experience} ")
     expect_equal(topic[[2]], "{February earthquake experience}")
     expect_equal(topic[[3]], "{aftermath of the earthquakes}")
+    expect_true(is.na(topic[[4]]))
 
 })
 
@@ -70,7 +72,7 @@ test_that("getMatchLabels works with count > 1", {
 
     expect_equal(length(labels$phonemes.2), 3)
     phonemes <- as.vector(labels$phonemes.2)
-    expect_equal(phonemes[[1]], "")
+    expect_true(is.na(phonemes[[1]]))
     expect_equal(phonemes[[2]], "wz")
     expect_equal(phonemes[[3]], "D@m")
 
